from shuup_testutils.cases import SnakeTestCase


class SnakeTestCaseTest(SnakeTestCase):
    is_set_up_class_works = False
    is_set_up_works = False

    @classmethod
    def set_up_class(cls):
        super().set_up_class()
        cls.is_set_up_class_works = True

    def set_up(self):
        self.is_set_up_works = True

    def test_set_up(self):
        self.assert_true(self.is_set_up_works)

    def test_set_up_class(self):
        self.assert_true(self.is_set_up_class_works)



import random
from decimal import Decimal
from typing import Union


# noinspection PyShadowingBuiltins
def decimal(min: Union[int, Decimal], max: Union[int, Decimal]) -> Decimal:
    return Decimal(random.randint(min, max))



import os
import subprocess
import importlib

from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.core.management import call_command
from shuup import configuration
from shuup.core.models import Shop
from shuup.xtheme import set_current_theme
from shuup.testing.factories import get_default_shop

from shuup_testutils.settings_base import SHUUP_DEFAULT_THEME


class IntegrationTestCase(StaticLiveServerTestCase):
    #: to determine the absolute module path
    python_module_name = None  # type: str
    #: relative to the module dir, example: 'static/{django_app_name}/'
    yarn_dir = None  # type: str
    #: relative to the yarn_dir
    protractor_conf = 'tests/protractor.conf.js'
    server_address = 'localhost:8082'

    @classmethod
    def setUpClass(cls):
        cls.set_up_class_before()
        super().setUpClass()
        cls.set_up_class_after()

    @classmethod
    def set_up_class_after(cls):
        cls._check_config()
        cls._set_yarn_directory()
        cls.init_shuup()

    @classmethod
    def set_up_class_before(cls):
        os.environ['DJANGO_LIVE_TEST_SERVER_ADDRESS'] = cls.server_address

    def run_protractor_tests(self):
        subprocess.run(['yarn', 'run', 'protractor', self.protractor_conf], check=True)

    @classmethod
    def _check_config(cls):
        if not cls.yarn_dir or not cls.python_module_name:
            raise TestCaseConfigError(
                'the attributes yarn_dir and module_name are required.'
            )

    # noinspection PyShadowingBuiltins
    @classmethod
    def _set_yarn_directory(cls):
        module = importlib.import_module(cls.python_module_name)
        module_path = os.path.dirname(module.__file__)
        os.chdir(module_path + '/' + cls.yarn_dir)

    @classmethod
    def init_shuup(cls):
        cls._create_shop_without_maintenance_mode()
        call_command('shuup_init')
        set_current_theme(SHUUP_DEFAULT_THEME)
        configuration.set(Shop.objects.first(), 'setup_wizard_complete', True)

    @classmethod
    def _create_shop_without_maintenance_mode(cls):
        get_default_shop()


class TestCaseConfigError(Exception):
    pass



import os

from shuup.utils.setup import Setup
from shuup_workbench.settings.utils import DisableMigrations


#-------------------------------------------------------------------------------
# directories
#-------------------------------------------------------------------------------

MEDIA_URL = '/media/'
STATIC_URL = '/static/'

#-------------------------------------------------------------------------------
# shuup settings
#-------------------------------------------------------------------------------

SHUUP_PRICING_MODULE = 'customer_group_pricing'

SHUUP_REGISTRATION_REQUIRES_ACTIVATION = False

SHUUP_SETUP_WIZARD_PANE_SPEC = [
    'shuup.admin.modules.shops.views:ShopWizardPane',
    'shuup.admin.modules.service_providers.views.PaymentWizardPane',
    'shuup.admin.modules.service_providers.views.CarrierWizardPane',
    'shuup.xtheme.admin_module.views.ThemeWizardPane',
    'shuup.admin.modules.content.views.ContentWizardPane',
    'shuup.admin.modules.sample_data.views.SampleObjectsWizardPane',
]

# shuup error_handlers are suppressing any debug info
SHUUP_ERROR_PAGE_HANDLERS_SPEC = None

SHUUP_DEFAULT_THEME = 'shuup.themes.classic_gray'

SHUUP_SIMPLE_SEARCH_LIMIT = 150

#-------------------------------------------------------------------------------
# django
#-------------------------------------------------------------------------------

DEBUG = True
SECRET_KEY = '0000'
SESSION_SERIALIZER = 'django.contrib.sessions.serializers.PickleSerializer'
SITE_ID = 1

#-------------------------------------------------------------------------------
# applications
#-------------------------------------------------------------------------------

INSTALLED_APPS_SHUUP_DEFAULT = [
    # django
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    # external apps that needs to be loaded before Shuup
    'easy_thumbnails',
    # shuup
    'shuup.addons',
    'shuup.admin',
    'shuup.api',
    'shuup.core',
    'shuup.default_tax',
    'shuup.front',
    'shuup.front.apps.auth',
    'shuup.front.apps.carousel',
    'shuup.front.apps.customer_information',
    'shuup.front.apps.personal_order_history',
    'shuup.front.apps.saved_carts',
    'shuup.front.apps.registration',
    'shuup.front.apps.simple_order_notification',
    'shuup.front.apps.simple_search',
    'shuup.front.apps.recently_viewed_products',
    'shuup.notify',
    'shuup.simple_cms',
    'shuup.customer_group_pricing',
    'shuup.campaigns',
    'shuup.simple_supplier',
    'shuup.order_printouts',
    'shuup.testing',
    'shuup.utils',
    'shuup.xtheme',
    'shuup.reports',
    'shuup.default_reports',
    'shuup.regions',
    'shuup.importer',
    'shuup.default_importer',
    SHUUP_DEFAULT_THEME,
    # external apps
    'bootstrap3',
    'django_countries',
    'django_jinja',
    'filer',
    'registration',
    'rest_framework',
    'rest_framework_swagger',
]

MIDDLEWARE_CLASSES = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'shuup.front.middleware.ProblemMiddleware',
    'shuup.front.middleware.ShuupFrontMiddleware',
]

#-------------------------------------------------------------------------------
# localization
#-------------------------------------------------------------------------------

LANGUAGES = [
    ('en', 'English'),
    ('fi', 'Finnish'),
    ('ja', 'Japanese'),
]
LANGUAGE_CODE = 'en'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True

PARLER_DEFAULT_LANGUAGE_CODE = 'en'
PARLER_LANGUAGES = {
    None: [dict(code=code, name=name) for (code, name) in LANGUAGES],
    'default': {
        'hide_untranslated': False,
    }
}

#-------------------------------------------------------------------------------
# templates
#-------------------------------------------------------------------------------

template_context_processors = [
    'django.contrib.auth.context_processors.auth',
    'django.contrib.messages.context_processors.messages',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.request',
    'django.core.context_processors.tz',
]

TEMPLATES = [
    {
        'BACKEND': 'django_jinja.backend.Jinja2',
        'APP_DIRS': True,
        'OPTIONS': {
            'match_extension': '.jinja',
            'context_processors': template_context_processors,
            'newstyle_gettext': True,
            'environment': 'shuup.xtheme.engine.XthemeEnvironment',
        },
        'NAME': 'jinja2',
    },
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': template_context_processors,
            'debug': DEBUG,
        }
    },
]

#-------------------------------------------------------------------------------
# URLs
#-------------------------------------------------------------------------------

ROOT_URLCONF = 'shuup_workbench.urls'
LOGIN_URL = '/login/'
LOGIN_REDIRECT_URL = '/'

#-------------------------------------------------------------------------------
# REST
#-------------------------------------------------------------------------------

REST_FRAMEWORK = {
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.LimitOffsetPagination',
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework_jwt.authentication.JSONWebTokenAuthentication',
    ),
    'DEFAULT_PERMISSION_CLASSES': (
        'shuup.api.permissions.ShuupAPIPermission',
    )
}

JWT_AUTH = {
    'JWT_ALLOW_REFRESH': True,
}

SWAGGER_SETTINGS = {
    'SUPPORTED_SUBMIT_METHODS': [
        'get',
    ],
}

#-------------------------------------------------------------------------------
# suppress deprecation warnings
#-------------------------------------------------------------------------------

LOGGING = {
    'version': 1,
    'formatters': {
        'verbose': {'format': '[%(asctime)s] (%(name)s:%(levelname)s): %(message)s'},
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        },
    },
    'loggers': {
        'shuup': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
    }
}

#-------------------------------------------------------------------------------
# test settings
#-------------------------------------------------------------------------------

DEFAULT_FROM_EMAIL = 'no-reply@example.com'
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

if os.environ.get('IS_SHUUP_TEST') is 'true':
    MIGRATION_MODULES = DisableMigrations()
    SOUTH_TESTS_MIGRATE = False


def update_globals():
    def configure(setup):
        setup.commit(globals())

    globals_new = Setup.configure(configure)
    globals().update(globals_new)



from typing import List

from bs4 import BeautifulSoup
from django.core.urlresolvers import reverse
from django.test import Client
from shuup.core.models import ProductMediaKind
from shuup.core.models import Shop
from shuup.testing.factories import get_default_shop, get_default_tax_class, \
    get_default_product_type, get_default_sales_unit
from shuup_utils.settings import LANG_CODES, DEFAULT_LANG

from shuup_testutils.cases.clients import AuthAdminClient


class ShuupProductAdminFormsGen:
    def __init__(
        self,
        product_pk: int = None,
        client: Client = None,
        shop: Shop = None,
    ):
        self._client = client or AuthAdminClient()
        self._shop = shop or get_default_shop()
        self._form = self._collect_form_from_dom(
            dom=self._get_product_edit_page_dom(product_pk),
        )

    def get_form(self) -> dict:
        return self._form
    
    # TODO rename the kwargs into name and value
    def set_field(self, field_name: str, field_value: str):
        self._form.update({field_name: field_value})
    
    def set_name_field(self, name_new: str):
        name_input_name = 'base-name__{}'.format(DEFAULT_LANG)
        self._form.update({name_input_name: name_new})
    
    def set_price_field(self, price: int):
        price_input_name = 'shop{}-default_price_value'.format(self._shop.pk)
        self._form.update({price_input_name: price})
    
    def set_barcode_field(self, barcode_new: str):
        barcode_input_name = 'base-barcode'
        self._form.update({barcode_input_name: barcode_new})
    
    def set_image_fields(self, image_ids: List[int]):
        forms_count_with_default_one = int(self._form['images-TOTAL_FORMS'])
        forms_count_actual = forms_count_with_default_one - 1
        number_of_new_images = len(image_ids)
        forms_count_new = forms_count_actual + number_of_new_images
        self._form.update({'images-TOTAL_FORMS': forms_count_new})
        
        for image_index, image_id in enumerate(image_ids):
            image_form_number = forms_count_actual + image_index
            self.set_image_field(image_number=image_form_number, image_id=image_id)
    
    def set_image_field(
        self,
        image_id: int,
        image_number: int = 0,
        is_logo: bool = False,
    ):
        image_form_prefix = 'images-{}-'.format(image_number)
        image_form_data = {
            image_form_prefix + 'kind': ProductMediaKind.IMAGE.value,
            image_form_prefix + 'file': image_id,
            image_form_prefix + 'ordering': image_number + 1,
            image_form_prefix + 'public': 'on',
            image_form_prefix + 'shops': self._shop.pk,
            image_form_prefix + 'is_primary': 'on' if is_logo else 'off',
            image_form_prefix + 'purchased': '',
            image_form_prefix + 'id': '',
        }
        
        for lang_code in LANG_CODES:
            title_input_name = image_form_prefix + 'title__{}'.format(lang_code)
            desc_input_name = image_form_prefix + 'description__{}'.format(lang_code)
            image_form_data.update({title_input_name: ''})
            image_form_data.update({desc_input_name: ''})
        
        self._form.update(image_form_data)
        
    def _collect_form_from_dom(self, dom: BeautifulSoup) -> dict:
        fields_input = self._collect_input_fields(dom)
        fields_select = self._collect_select_fields(dom)
        fields_select_multiple = self._collect_select_multiple_fields(dom)
        field_shop_visibility = self._collect_shop_visibility_field()
        
        form = {
            **fields_input, **fields_select, **fields_select_multiple,
            **field_shop_visibility,
        }
        self._fix_wrong_default_images_counter(form)
        return form
    
    def _get_product_edit_page_dom(self, product_pk: int = None) -> BeautifulSoup:
        is_user_wants_to_edit_existing_product = product_pk
        if is_user_wants_to_edit_existing_product:
            product_form_url = reverse(
                viewname='shuup_admin:shop_product.edit',
                kwargs={'pk': product_pk},
            )
        else:
            product_form_url = reverse('shuup_admin:shop_product.new')
        self._init_client()
        self._init_shuup()
        edit_page_response = self._client.get(product_form_url)
        edit_page_dom = BeautifulSoup(edit_page_response.content)
        return edit_page_dom

    def _init_client(self):
        if type(self._client) is AuthAdminClient:
            self._client.init()
            self._client.login()

    def _init_shuup(self):
        """It'll create a set of required model for shuup."""
        get_default_tax_class()
        get_default_product_type()
        get_default_sales_unit()

    def _collect_input_fields(self, dom: BeautifulSoup) -> dict:
        inputs = {}
        form_inputs = dom.find_all('input', {'name': True})
        for form_input in form_inputs:
            input_name = form_input['name']
            input_value = form_input.get('value', '')
            inputs.update({input_name: input_value})
        return inputs
    
    def _collect_select_fields(self, dom: BeautifulSoup) -> dict:
        select_nodes = dom.find_all(
            name='select',
            attrs={'name': True, 'multiple': False},
        )
        select_fields = {}
        for select_node in select_nodes:
            select_option_selected = select_node.find('option', selected=True)
            if select_option_selected is not None:
                select_value = select_option_selected['value']
            else:
                select_value = ''
            
            select_name = select_node['name']
            select_fields.update({select_name: select_value})
        return select_fields
    
    def _collect_select_multiple_fields(self, dom: BeautifulSoup) -> dict:
        select_nodes = dom.find_all(
            name='select',
            attrs={'name': True, 'multiple': True},
        )
        select_fields = {}
        for select_node in select_nodes:
            select_options = select_node.find_all('option', selected=True)
            select_values = []
            for select_option in select_options:
                select_values.append(select_option['value'])
            
            select_name = select_node['name']
            
            select_fields.update({select_name: select_values})
        return select_fields
    
    def _collect_shop_visibility_field(self) -> dict:
        # I can't catch the input with BeautifulSoup
        input_name = 'shop{shop_pk}-visible'.format(shop_pk=self._shop.pk)
        input_value = 'on'
        shop_visibility_field = {input_name: input_value}
        return shop_visibility_field
    
    def _fix_wrong_default_images_counter(self, form: dict) -> dict:
        if not ('images-TOTAL_FORMS' in form):
            return form
        images_counter_wrong = int(form['images-TOTAL_FORMS'])
        images_counter_fixed = images_counter_wrong - 1
        form.update({'images-TOTAL_FORMS': images_counter_fixed})
        return form
